USE adventureworks;
GO

ALTER PROCEDURE dbo.sp1
(
@CustomerID INT,
@Log BIT = 1
)
AS
/*
Input param with default.

* Use a default value if it's going to be run one way most
of the time. This saves typing, network bandwidth, and helps
the user by not making them know the value for every param.

****It also helps maintain your contracts.

Log table:
CREATE TABLE QueryLog
(
ExecDate DATETIME,
CustomerID INT,
RowCT int
)

Returns data
dbo.sp1 140
No data
dbo.sp1 1234
*/

DECLARE @ExecDate DATETIME,
		@CT int; --Holds count of records to compare.
SET @ExecDate = GETDATE();

CREATE TABLE #T1
(
OrderQty smallint,
NAME nvarchar(50),
ListPrice money
)

INSERT #T1 (OrderQty, NAME, ListPrice)
SELECT OrderQty,Name,ListPrice
  FROM SalesLT.SalesOrderHeader 
  INNER JOIN SalesLT.SalesOrderDetail
          ON SalesOrderDetail.SalesOrderID = SalesOrderHeader.SalesOrderID
  INNER JOIN SalesLT.Product
          ON SalesOrderDetail.ProductID=Product.ProductID
WHERE CustomerID = @CustomerID;

SET @CT = (SELECT COUNT (*) FROM #T1);

IF @CT > 0
	BEGIN --Always comment 1
		SELECT @ExecDate AS ExecDate, * FROM T1;
	END --Always comment 1
ELSE
	BEGIN --Always comment 2
		SELECT NULL AS ExecDate, NULL AS OrderQty, 'No data' AS Name, NULL AS ListPrice;
	END --Always comment 2

IF @Log = 1
	BEGIN
		INSERT  QueryLog(ExecDate, CustomerID, RowCT)
		SELECT @ExecDate, @CustomerID, @CT
	END

