---------------------------------------------------------------------------------															          
---------------------------------------------------------------------------------
------------------ OUTPUT Param Usage -------------------------------------------
---------------------------------------------------------------------------------
---------------------------------------------------------------------------------
/*
Now that we know all about the syntax,
it's time to take a look at some practical
usage for output params.
*/

USE SPWorkshop;
GO
ALTER PROCEDURE dbo.DBNameGet
AS

SELECT TOP(1) DBName
FROM AdvancedScripting.dbo.Databases
WHERE RecoveryModel = 'Simple';
GO

ALTER PROCEDURE dbo.TempTable

AS

/*
Shows how to get values back from an SP
w/o output params.
This uses insert/exec which only works with SPs that return
resultsets (not output params), and the cols must match exactly.

Sample Call:
dbo.TempTable
*/
DECLARE @DBName NVARCHAR(100);
CREATE TABLE #DB
(
DBName NVARCHAR(100)
);

INSERT #DB(DBName)
EXEC dbo.DBNameGet;

SET @DBName = (SELECT DBName FROM #DB);
SELECT @DBName AS DBName;
GO
--Make the call.
EXEC dbo.TempTable;

----Now let's use an OUTPUT param.
GO
ALTER PROCEDURE dbo.DBNameGet
(
@DBName NVARCHAR(100) OUTPUT
)
AS

SET @DBName = (SELECT TOP(1) DBName
FROM AdvancedScripting.dbo.Databases
WHERE RecoveryModel = 'Simple');
GO

ALTER PROCEDURE dbo.TempTable

AS

/*
Shows hw to get values back from an SP
w/o output params.

Sample Call:
dbo.TempTable
*/
DECLARE @DBName NVARCHAR(100);
EXEC dbo.DBNameGet @DBName = @DBName OUTPUT;
SELECT @DBName AS DBName;
GO
--Make the call.
EXEC dbo.TempTable;

GO
----And it works just as well with
----2 params.
ALTER PROCEDURE dbo.DBNameGet
(
@DBName NVARCHAR(100) OUTPUT,
@ID INT OUTPUT
)
AS

SET @DBName = (SELECT TOP(1) DBName
FROM AdvancedScripting.dbo.Databases
WHERE RecoveryModel = 'Simple');

SET @ID = 15;
GO

ALTER PROCEDURE dbo.TempTable

AS

/*
Shows hw to get values back from an SP
with output params.

Sample Call:
dbo.TempTable
*/
DECLARE @DBName NVARCHAR(100),
		@ID INT;
EXEC dbo.DBNameGet @DBName = @DBName OUTPUT,
			   @ID = @ID OUTPUT;
SELECT @DBName AS DBName, @ID AS ID;
GO
--Make the call.
EXEC dbo.TempTable;

GO

----This only works with single records
----though. You can have as many values
----as you like, but they must account
----for a single row of data.
----So if you need to return mult rows
----then you need to use the #table
----method.
--**I should note that these aren't your
--only 2 choices, but they'll do for now.





