
/*
This is a good example of an SP that handles more than one
module.  It's not exactly generic, but it handles the diff
situations I'm interested in.
*/

ALTER PROCEDURE [Minion].[DBMaintDBSizeGet]
(
@Module VARCHAR(20),
@OpName VARCHAR(20) OUTPUT,
@DBName VARCHAR(400),
@DBSize DECIMAL(18, 2) OUTPUT
)

AS

SET NOCOUNT ON;
DECLARE	@Version DECIMAL(3,1),
		@SQL nvarchar(2000),
		@DBMaintDB VARCHAR(400);

SET @DBMaintDB = DB_NAME();

CREATE TABLE #DBSize
(
DBName VARCHAR(200),
Size FLOAT,
SpaceUsed FLOAT,
DataSpaceUsed FLOAT,
IndexSpaceUsed FLOAT
);


SELECT @Version = Version 
FROM Minion.DBMaintSQLInfoGet()

-----------------------------------------------------
-------------------BEGIN AUTO------------------------
-----------------------------------------------------

IF @Module = 'CHECKDB'
	BEGIN --CheckDB Op
DECLARE @CheckDBAutoSettingLevel INT,
		@AutoThresholdMethod VARCHAR(20),
		@AutoThresholdType VARCHAR(20),
		@AutoThresholdValue INT;

----0 = MinionDefault, > 0 = DBName.
IF UPPER(@OpName) = 'AUTO' OR @OpName IS NULL
	BEGIN --AUTO Params
			SET @CheckDBAutoSettingLevel = (
									SELECT COUNT(*)
									FROM Minion.CheckDBSettingsAutoThresholds
									WHERE
										DBName = @DBName
										AND IsActive = 1
								);

                        IF @CheckDBAutoSettingLevel = 0
                            BEGIN --@TuningTypeLevel = 0
                                SELECT TOP 1
                                        @AutoThresholdMethod = ThresholdMethod,
										@AutoThresholdType = ThresholdType,
										@AutoThresholdValue = ThresholdValue
                                    FROM
                                        Minion.CheckDBSettingsAutoThresholds
                                    WHERE
                                        DBName = 'MinionDefault'
                                        AND IsActive = 1

                            END --@TuningTypeLevel = 0

                        IF @CheckDBAutoSettingLevel > 0
                            BEGIN --@TuningTypeLevel > 0
                                SELECT TOP 1
                                        @AutoThresholdMethod = ThresholdMethod,
										@AutoThresholdType = ThresholdType,
										@AutoThresholdValue = ThresholdValue
                                    FROM
                                        Minion.CheckDBSettingsAutoThresholds
                                    WHERE
                                        DBName = @DBName
                                        AND IsActive = 1
                            END --@TuningTypeLevel > 0
END --AUTO Params
END --CheckDB Op
-----------------------------------------------------
-------------------END AUTO--------------------------
-----------------------------------------------------


------------------------------------------------------------------------
------------------------------------------------------------------------
-------------------------BEGIN DBSize-----------------------------------
------------------------------------------------------------------------
------------------------------------------------------------------------

/*
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
the reason we have the Auto table is because the thresholds table holds values for each op type and we
wouldn't have a solid place to keep the values. So where do we do it, or checkdb or checktable, etc?
So this is a clean settings table to say when to switch from checkdb to checktable w/o any of the hassle
of figuring out which row to use for each DB.
Also, we should prob rename checkdbthresholds to checkdbsnapsotsettings. it's more fitting.
in our auto table we prob need to give the unit of measure for each value so it's clear what we're doing.
we're also currently using the threshold setting from Settings and I think that needs to stop.
We also need to add in table count to the auto table so we can do more than just size.
*/

CREATE TABLE #TSizeGet
    (
        ID INT IDENTITY(1, 1),
        col1 VARCHAR(MAX) COLLATE DATABASE_DEFAULT
    )

DECLARE @SpaceType VARCHAR(20),
		@DBSizeCMD VARCHAR(8000)--,
		--@DBSize DECIMAL(18, 2);
		

---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
----------------------------------BEGIN Maint Type-------------------------------------------------
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
----Here we're getting space the same way we did in MB.
BEGIN --DBSize
IF @Module = 'CHECKDB'
	BEGIN --CheckDB Op
		IF UPPER(@AutoThresholdMethod) = 'SIZE'
			BEGIN
				SET @SpaceType = @AutoThresholdType;
	END
	END --CheckDB Op
                        IF @SpaceType IS NULL
                            BEGIN
                                SET @SpaceType = 'DataAndIndex';
                            END

---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
----------------------------------END Maint Type---------------------------------------------------
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------



---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
----------------------------------BEGIN Get Size Data----------------------------------------------
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------

--Here we're avoiding the Insert/Exec syntax so we can use this in mult. SPs.
--We don't need to keep this data long at all so we delete it as soon as we get it into our #table.

--SET @SQL = 'USE [' + @DBName + ']; SET NOCOUNT ON; INSERT [' + @DBMaintDB + '].Minion.DBMaintDBSizeTemp (DBName,Size,SpaceUsed,DataSpaceUsed,IndexSpaceUsed) SELECT DB_Name() AS DBName,(SELECT SUM(CAST(df.size as float)) FROM ' + @DBName + '.sys.database_files AS df WHERE df.type in ( 0, 2, 4 ) )*8 AS [Size], SUM(a.total_pages)*8 AS [SpaceUsed], SUM(CASE When it.internal_type IN (202,204,207,211,212,213,214,215,216) Then 0 When a.type <> 1 Then a.used_pages	When p.index_id < 2 Then a.data_pages	Else 0	END)*8 AS [DataSpaceUsage], SUM(a.used_pages)*8 AS [IndexSpaceTotal] FROM sys.partitions p join sys.allocation_units a on p.partition_id = a.container_id left join sys.internal_tables it on p.object_id = it.object_id;'
SET @SQL = 'SELECT DB_Name() AS DBName,(SELECT SUM(CAST(df.size as float)) FROM ' + @DBName + '.sys.database_files AS df WHERE df.type in ( 0, 2, 4 ) )*8 AS [Size], SUM(a.total_pages)*8 AS [SpaceUsed], SUM(CASE When it.internal_type IN (202,204,207,211,212,213,214,215,216) Then 0 When a.type <> 1 Then a.used_pages	When p.index_id < 2 Then a.data_pages	Else 0	END)*8 AS [DataSpaceUsage], SUM(a.used_pages)*8 AS [IndexSpaceTotal] FROM sys.partitions p join sys.allocation_units a on p.partition_id = a.container_id left join sys.internal_tables it on p.object_id = it.object_id;'

----		EXEC (@SQL)
----INSERT #DBSize
----		(DBName, Size, SpaceUsed, DataSpaceUsed, IndexSpaceUsed)
----SELECT TOP 1 * FROM Minion.DBMaintDBSizeTemp WHERE DBName = @DBName;

----DELETE Minion.DBMaintDBSizeTemp WHERE DBName = @DBName;

create table #SizeResults(col1 nvarchar(2000));

DECLARE @SizeCMD VARCHAR(2000),
		@PreCMD varchar(5000),
		@ServerInstance varchar(200),
		@Port varchar(6),
		@TotalCMD varchar(8000),
		@IsClustered BIT,
		@SizeGetSqlCMD NVARCHAR(400),
		@DBIsInAG BIT,
		@IsPrimaryReplica BIT,
		@AG2ndaryReadMode VARCHAR(10),
		@IsLocal BIT,
		@PrimaryNode VARCHAR(200);

IF @Version >= 12
	BEGIN
		EXEC [Minion].[MMAGInfo]
		@DBName = @DBName,
		@DBIsInAG = @DBIsInAG OUTPUT,
		@IsPrimaryReplica = @IsPrimaryReplica OUTPUT,
		--@IsPreferredBackup = @IsPreferredBackup OUTPUT,
		@AG2ndaryReadMode = @AG2ndaryReadMode OUTPUT,
		--@AGRole = @AGRole OUTPUT,
		--@AGName = @AGName OUTPUT,
		@PrimaryNode = @PrimaryNode;
--SELECT @DBIsInAG = DBIsInAG,
--		@IsPrimaryReplica = IsPrimaryReplica,
--		@AG2ndaryReadMode = AG2ndaryReadMode,
--		@PrimaryNode = PrimaryNode
--FROM [Minion].[MMAGInfo](@DBName);
	END
ELSE
	BEGIN
		SET @DBIsInAG = 0;
		SET @IsPrimaryReplica = 1;
		SET @AG2ndaryReadMode = '';
		SET @PrimaryNode = '';
	END
-------------------------------------
---------BEGIN Set ServerName--------
-------------------------------------
IF @DBIsInAG = 1 AND @AG2ndaryReadMode <> 'NO'
	BEGIN
		SET @ServerInstance = @@SERVERNAME;
	END
IF @DBIsInAG = 1 AND @AG2ndaryReadMode = 'NO'
	BEGIN
		SET @ServerInstance = @PrimaryNode;
	END
-------------------------------------
---------END Set ServerName----------
-------------------------------------

----We're going under the premise that if we're not
----on the primary and the DB doesn't accept read conns,
----that we'll need to conn to the primary box to get the
----size. So here we have to set IsLocal = 0 or the
----function may translate it to an in-memory conn.
IF @IsPrimaryReplica = 0 AND @AG2ndaryReadMode = 'NO'
	BEGIN
		SET @IsLocal = 0;
	END
ELSE
	BEGIN
		SET @IsLocal = 1;
	END

SELECT @SizeGetSqlCMD = SqlCMD 
FROM [Minion].[MMCmdServerNameGet]('Backup', 0, @ServerInstance, @Port, @IsLocal, @AG2ndaryReadMode);

-------------------------------------
---------BEGIN Full Read-------------
-------------------------------------
----DB is not in an AG so we should be able to simply query it.
----Or if it's in an AG and open to all queries.
IF @DBIsInAG = 0 OR @AG2ndaryReadMode = 'All'
	BEGIN	
		SET @SQL = 'USE [' + @DBName + ']; INSERT [' + @DBMaintDB + '].Minion.DBMaintDBSizeTemp (DBName,Size,SpaceUsed,DataSpaceUsed,IndexSpaceUsed) SELECT DB_Name() AS DBName,
(SELECT SUM(CAST(df.size as float)) FROM sys.database_files AS df WHERE df.type in ( 0, 2, 4 ) )*8 AS [Size],
SUM(a.total_pages)*8 AS [SpaceUsed],
SUM(CASE When it.internal_type IN (202,204,207,211,212,213,214,215,216) Then 0 When a.type <> 1 Then a.used_pages	When p.index_id < 2 Then a.data_pages	Else 0	END)*8 AS [DataSpaceUsage],
SUM(a.used_pages)*8 AS [IndexSpaceTotal]
FROM
sys.partitions p join sys.allocation_units a on p.partition_id = a.container_id left join sys.internal_tables it on p.object_id = it.object_id;'
		EXEC (@SQL)
	END
-------------------------------------
---------END Full Read---------------
-------------------------------------

-------------------------------------
---------BEGIN AG ReadIntent---------
-------------------------------------
IF @IsPrimaryReplica = 0 AND @AG2ndaryReadMode = 'READ_ONLY'
	BEGIN
		SELECT @SizeGetSqlCMD = SqlCMD 
		FROM [Minion].[MMCmdServerNameGet]('Backup', 0, @ServerInstance, @Port, @IsLocal, @AG2ndaryReadMode);
--USE [' + @DBName + ']; 
		SET @SQL = 'INSERT [' + @DBMaintDB + '].Minion.DBMaintDBSizeTemp (DBName,Size,SpaceUsed,DataSpaceUsed,IndexSpaceUsed) SELECT DB_Name() AS DBName, (SELECT SUM(CAST(df.size as float)) FROM sys.database_files AS df WHERE df.type in ( 0, 2, 4 ) )*8 AS [Size], SUM(a.total_pages)*8 AS [SpaceUsed], SUM(CASE When it.internal_type IN (202,204,207,211,212,213,214,215,216) Then 0 When a.type <> 1 Then a.used_pages	When p.index_id < 2 Then a.data_pages	Else 0	END)*8 AS [DataSpaceUsage], SUM(a.used_pages)*8 AS [IndexSpaceTotal] FROM sys.partitions p join sys.allocation_units a on p.partition_id = a.container_id left join sys.internal_tables it on p.object_id = it.object_id;'
		SET @PreCMD = @SizeGetSqlCMD
			+ ' -d "' + @DBName + '"' 
		SET @TotalCMD = @PreCMD
			+ ' -Q "' + @SQL + '" ';
		--PRINT @TotalCMD
		DECLARE @FileDeleteErrors VARCHAR(max);
		INSERT #SizeResults
				(col1)
		EXEC xp_cmdshell @TotalCMD;

INSERT #DBSize
		(DBName, Size, SpaceUsed, DataSpaceUsed, IndexSpaceUsed)
SELECT TOP 1  DBName,
              Size,
              SpaceUsed,
              DataSpaceUsed,
              IndexSpaceUsed FROM Minion.DBMaintDBSizeTemp WHERE DBName = @DBName;
DELETE Minion.DBMaintDBSizeTemp WHERE DBName = @DBName;
END
--SELECT * FROM #SizeResults
--SELECT * FROM Minion.DBMaintDBSizeTemp
-------------------------------------
---------END AG ReadIntent-----------
-------------------------------------
--SELECT * FROM Minion.DBMaintDBSizeTemp

-------------------------------------
---------BEGIN AG No Read------------
-------------------------------------
IF @IsPrimaryReplica = 0 AND @AG2ndaryReadMode = 'NO'
	BEGIN
	
		CREATE TABLE #RemoteResults (ID INT IDENTITY(1,1), col1 VARCHAR(max) COLLATE DATABASE_DEFAULT);

		DECLARE @RemoteSQL VARCHAR(4000),
				@RemoteCMD VARCHAR (8000),
				@RemoteResultsError VARCHAR(MAX);
		SET @RemoteCMD = 'EXEC xp_cmdshell ''powershell "';
		SET @RemoteSQL = '$RemoteBox = """' + @ServerInstance + CASE WHEN (@Port IS NOT NULL AND @Port <> '') THEN ','  ELSE '' END
				+ ISNULL(CAST(@Port AS VARCHAR(10)), '') + '""";'
		+ '$MainBox = """' + @@SERVERNAME + CASE WHEN (@Port IS NOT NULL AND @Port <> '') THEN ','  ELSE '' END
				+ ISNULL(CAST(@Port AS VARCHAR(10)), '') + '""";'
		+ '$Table = """Minion.DBMaintDBSizeTemp""";'
		+ '$DB = """' + @DBMaintDB + '""";'
		+ '$RemoteQuery = ''''SELECT ' + '''''''''' + @DBName  + '''''''''' +  ' AS DBName,(SELECT SUM(CAST(df.size as float)) FROM [' + @DBName + '].sys.database_files AS df WHERE df.type in ( 0, 2, 4 ) )*8 AS [Size], SUM(a.total_pages)*8 AS [SpaceUsed], SUM(CASE When it.internal_type IN (202,204,207,211,212,213,214,215,216) Then 0 When a.type <> 1 Then a.used_pages	When p.index_id < 2 Then a.data_pages	Else 0	END)*8 AS [DataSpaceUsage], SUM(a.used_pages)*8 AS [IndexSpaceTotal] FROM sys.partitions p join sys.allocation_units a on p.partition_id = a.container_id left join sys.internal_tables it on p.object_id = it.object_id;'''';'
		--+ CONVERT(VARCHAR(30), @ExecutionDateTime, 21) + '''''' + 'AND DBName = ''''' + @DBName + ''''';""";'
		+ '$RemoteConnString = """Data Source=$RemoteBox;Initial Catalog=master;Integrated Security=SSPI;""";'
		+ '$RemoteConn  = New-Object System.Data.SqlClient.SQLConnection($RemoteConnString);'
		+ '$SqlCMD = New-Object system.Data.SqlClient.SqlCommand($RemoteQuery, $RemoteConn);'
		+ '$RemoteConn.Open();'
		+ '[System.Data.SqlClient.SqlDataReader] $SqlReader = $SqlCMD.ExecuteReader();'
		+ '$MainConnString = """Data Source=$MainBox;Initial Catalog=$DB;Integrated Security=SSPI;""";'
		+ '$BC = New-Object Data.SqlClient.SqlBulkCopy($MainConnString, [System.Data.SqlClient.SqlBulkCopyOptions]::Default);'
		+ '$BC.DestinationTableName = $Table;' 
		+ '$BC.WriteToServer($sqlReader); ' 
		+ '$SqlReader.close();' 
		+ '$RemoteConn.Close();' 
		+ '$RemoteConn.Dispose();' 
		+ '$BC.Close();';

		SET @RemoteCMD = @RemoteCMD + @RemoteSQL;
		SET @RemoteCMD = @RemoteCMD + '"''';
----PRINT @RemoteCMD
			INSERT #RemoteResults(col1)
			EXEC (@RemoteCMD)
	END
-------------------------------------
---------END AG No Read--------------
-------------------------------------
INSERT #DBSize
		(DBName, Size, SpaceUsed, DataSpaceUsed, IndexSpaceUsed)
SELECT TOP 1  DBName,
              Size,
              SpaceUsed,
              DataSpaceUsed,
              IndexSpaceUsed FROM Minion.DBMaintDBSizeTemp WHERE DBName = @DBName;
--SELECT TOP 1  * FROM Minion.DBMaintDBSizeTemp;
--select * from #DBSize
DELETE Minion.DBMaintDBSizeTemp WHERE DBName = @DBName;
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
----------------------------------END Get Size Data------------------------------------------------
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------


---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
----------------------------------BEGIN Set Size Type----------------------------------------------
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------

			IF @SpaceType = 'File'
				BEGIN	
					SET @DBSize = (SELECT CAST(Size/1024.0/1024.0 AS DECIMAL(18,2)) FROM #DBSize);			 
				END

			IF @SpaceType = 'Data'
				BEGIN
					SET @DBSize = (SELECT CAST(Size/1024.0/1024.0 AS DECIMAL(18,2)) FROM #DBSize);
				END

			IF @SpaceType = 'DataAndIndex'
				BEGIN --DataAndIndex
					SET @DBSize = (SELECT CAST(Size/1024.0/1024.0 AS DECIMAL(18,2)) FROM #DBSize);
				END --DataAndIndex

---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
----------------------------------END Set Size Type------------------------------------------------
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------


-------------------------------------------------------------------
--------------------BEGIN Size Format------------------------------
-------------------------------------------------------------------
----If something happens with the above call then the @DBSize var won't be able to be assigned so this is an excellent place to put in the error handling.
							--Sometimes on int'l systems we get a , back from PS so we have to change it back to a decimal point.
							
							--(SELECT REPLACE(col1, ',', '.') FROM #TSizeGet)
							SET @DBSize = REPLACE(@DBSize, ',', '.');
							DROP TABLE #DBSize;

----If DBSize is under 0 it won't trigger the NumberOfFiles so it has to be at least 0.
                        IF @DBSize < 1
                            BEGIN
                                SET @DBSize = 1;
                            END
-------------------------------------------------------------------
--------------------BEGIN Size Format------------------------------
-------------------------------------------------------------------


-------------------------------------------------------------------
--------------------BEGIN Op Type----------------------------------
-------------------------------------------------------------------
	IF @Module = 'CHECKDB'
		BEGIN --CheckDB Op
			IF @OpName IS NULL OR UPPER(@OpName) = 'AUTO'
			BEGIN --@Op IS NULL
				IF @DBSize >= @AutoThresholdValue
					BEGIN
						SET @OpName = 'CHECKTABLE'
					END

				IF @DBSize < @AutoThresholdValue
					BEGIN
						SET @OpName = 'CHECKDB'
					END
			 END--@Op IS NULL
		 END --CheckDB Op
-------------------------------------------------------------------
--------------------END Op Type------------------------------------
-------------------------------------------------------------------

 END --DBSize
------------------------------------------------------------------------
------------------------------------------------------------------------
-------------------------END DBSize-------------------------------------
------------------------------------------------------------------------
------------------------------------------------------------------------



