


/*
This is an excellent example of a couple things.
1. It's a real-world example of using output params.
2. It uses the component model.

We haven't talked a lot about the component model.  It's the only sane way to
keep code, but it's not written in stone.  Most of it boils down to preference.
I like to make components if I use a piece of code twice. 
*/
ALTER PROCEDURE [Minion].[MMAGInfo]
(
@DBName NVARCHAR(400),
@DBIsInAG BIT = 0 OUTPUT,
@IsPrimaryReplica BIT = 1 OUTPUT,
@IsPreferredBackup BIT = 1 OUTPUT,
@AGRole VARCHAR(10) = NULL OUTPUT,
@AGName NVARCHAR(100) = NULL OUTPUT,
@PrimaryNode VARCHAR(200) = NULL OUTPUT,
@AG2ndaryReadMode varchar(10) = 'All' OUTPUT
)
AS

/*

Sample call:
DECLARE @DBIsInAG TINYINT,
		@IsPrimaryReplica TINYINT,
		@IsPreferredBackup TINYINT,
		@AG2ndaryReadMode VARCHAR(10),
		@AGRole VARCHAR(10),
		@AGName nvarchar(100),
		@PrimaryNode VARCHAR(200),
		@DBName NVARCHAR(400) = 'aaa'
		;
EXEC [Minion].[MMAGInfo2]
@DBName = 'BIT2',
@DBIsInAG = @DBIsInAG OUTPUT,
@IsPrimaryReplica = @IsPrimaryReplica OUTPUT,
@IsPreferredBackup = @IsPreferredBackup OUTPUT,
@AG2ndaryReadMode = @AG2ndaryReadMode OUTPUT,
--@AGRole = @AGRole OUTPUT,
--@AGName = @AGName OUTPUT,
@PrimaryNode = @PrimaryNode;
SELECT 
@DBName AS DBName,
@DBIsInAG AS DBIsInAG, 
@IsPrimaryReplica AS IsPrimaryReplica,
@IsPreferredBackup AS IsPreferredBackup,
@AG2ndaryReadMode AS AG2ndaryReadMode,
@AGRole AS AGRole,
@AGName AS AGName,
@PrimaryNode AS PrimaryNode;
*/

DECLARE @Version DECIMAL(3,1);

---------------------------------------------------------------------------------
------------------ BEGIN Get Version Info----------------------------------------
---------------------------------------------------------------------------------
SELECT 
	@Version = [Version]
FROM Minion.MMSQLInfoGet();

------------------------------------
----------BEGIN DBIsInAG------------
------------------------------------
IF @Version >= 11
	BEGIN --@Version
		DECLARE @DynamicCMD NVARCHAR(1000),
				@DynamicResults SMALLINT;
		SET @DynamicCMD = N'SELECT @DynamicResults = COUNT(1) from sys.databases with(nolock) where Name = @DBName AND replica_id IS NOT NULL';
		EXEC sp_executesql @DynamicCMD, N'@DBName nvarchar(400), @DynamicResults smallint OUTPUT', @DBName, @DynamicResults = @DBIsInAG OUTPUT;
	END	--@Version									          
IF @DBIsInAG > 1
	BEGIN
		SET @DBIsInAG = 1;
	END 

IF @Version < 11
	BEGIN --@Version
		SET @DBIsInAG = 0;
	END
------------------------------------
----------END DBIsInAG--------------
------------------------------------

------------------------------------
-------BEGIN IsPreferredBackup------
------------------------------------
IF @DBIsInAG = 1
	BEGIN
		SET @IsPreferredBackup = sys.fn_hadr_backup_is_preferred_replica(@DBName);
	END
------------------------------------
-------END IsPreferredBackup--------
------------------------------------

------------------------------------
-------BEGIN PrimaryNode------------
------------------------------------
IF @Version >= 11
	BEGIN --@Version
SELECT
@AGName = AG.name, -- AS GroupName,
@PrimaryNode = ISNULL(agstates.primary_replica, @@SERVERNAME), -- AS PrimaryServer,
@AGRole = CASE WHEN arstates.role = 1 THEN 'Primary'
	 WHEN arstates.role = 2 THEN 'Secondary'  
	 WHEN arstates.role = 3 THEN 'None'
	 END,
@IsPrimaryReplica = CASE WHEN arstates.role = 1 THEN 1
				  ELSE 0
			 END,
@AG2ndaryReadMode = AR.secondary_role_allow_connections_desc
FROM master.sys.availability_groups AS AG
LEFT OUTER JOIN master.sys.dm_hadr_availability_group_states as agstates
   ON AG.group_id = agstates.group_id
INNER JOIN master.sys.availability_replicas AS AR
   ON AG.group_id = AR.group_id
INNER JOIN master.sys.dm_hadr_availability_replica_states AS arstates
   ON AR.replica_id = arstates.replica_id AND arstates.is_local = 1
INNER JOIN master.sys.dm_hadr_database_replica_cluster_states AS drcs
   ON arstates.replica_id = drcs.replica_id
LEFT OUTER JOIN master.sys.dm_hadr_database_replica_states AS dbrs
   ON drcs.replica_id = dbrs.replica_id AND drcs.group_database_id = dbrs.group_database_id
WHERE drcs.database_name = @DBName;
	  --AND arstates.role = 1;
	  END --@Version
------------------------------------
-------END PrimaryNode--------------
------------------------------------

----We want to always return a PrimaryNode because
----The SPs can compare it with the current node and this
----makes it easier in the SP if we return a value.

SET @IsPrimaryReplica = ISNULL(@IsPrimaryReplica, 1); 
SET @IsPreferredBackup = ISNULL(@IsPreferredBackup, 1); 
SET @PrimaryNode = ISNULL(@PrimaryNode, @@SERVERNAME); 
SET @AG2ndaryReadMode = ISNULL(@AG2ndaryReadMode, 'All');




